##########################################################################
# SmartPrintNG - high-quality export of Plone content to
# PDF, RTF, ODT, WML and DOCX
#
# (C) 2007, ZOPYX Ltd & Co. KG, Tuebingen, Germany
##########################################################################


from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import getFSVersionTuple

def install(self, reinstall=False):

    
    mtool = getToolByName(self, 'portal_migration')
    plone_version = mtool.getFileSystemVersion()
    if plone_version.startswith('3.0'):
        version = '3.0'
    elif plone_version.startswith('3.1'):
        version = '3.1'
    else:
        raise RuntimeError('Unsupported Plone version %s' % plone_version)

    tool=getToolByName(self, "portal_setup")

    if version == '3.0':
        tool.runAllImportStepsFromProfile(
                "profile-Products.SmartPrintNG:smartprintng",
                purge_old=False)
    else:
        tool.runAllImportStepsFromProfile(
                "profile-Products.SmartPrintNG:default",
                purge_old=False)



##########################################################################
# SmartPrintNG - high-quality export of Plone content to
# PDF, RTF, ODT, WML and DOCX
#
# (C) 2007, ZOPYX Ltd & Co. KG, Tuebingen, Germany
##########################################################################

"""
Factory for converters
"""

from zope.interface import implements, implementedBy
from zope.component.interfaces import IFactory

from zopyx.convert import Converter

class Factory:
    """ zopyx.convert.Converter factory """
    
    implements(IFactory)

    def __init__(self, klass):
        self._klass = klass

    def __call__(self, *args, **kw):
        return self._klass(*args, **kw)

    def getInterfaces(self):
        return implementedBy(self._klass)

Factory = Factory(Converter)



# -*- coding: iso-8859-15 -*-

##########################################################################
# z3c.sqlalchemy - A SQLAlchemy wrapper for Python/Zope
#
# (C) Zope Corporation and Contributor
# Written by Andreas Jung for Haufe Mediengruppe, Freiburg, Germany
# and ZOPYX Ltd. & Co. KG, Tuebingen, Germany
##########################################################################


"""
Tests, tests, tests.........
"""

import unittest

from Products.SmartPrintNG.converter.manipulation import breakIntoPages, enumerateLinksInHtml, getLinksInHtml 
from Products.SmartPrintNG.converter.manipulation import removeLinks

html = """
<a href="http://www.heise.de" />HEISE</A>
<a href="@@control-panel" > Control Panel</a>
<a HREF="#anchor">
<a href="http://www.heise.de#anchor" />HEISE</A>
<A   HrEF="/foo/bar" />foo bar</A>
<a href="http://foo.com"><em>bar</em></a>
"""


class ManipulationTests(unittest.TestCase):

    def testGeLinks(self):
        links = getLinksInHtml(html)    
        self.assertEqual(links, ['http://www.heise.de',
                                 'http://www.heise.de#anchor',
                                 '/foo/bar',                  
                                 'http://foo.com',
                                ])

    def testEnumerateLinks(self):
        links = getLinksInHtml(html)
        html2 = enumerateLinksInHtml(html, links)
        self.assertEqual('HEISE [1]' in html2, True)
        self.assertEqual('HEISE [2]' in html2, True)
        self.assertEqual('foo bar [3]' in html2, True)
        self.assertEqual('bar [4]' in html2, True)


    def testBreakIntoPages(self):
        html = """<h1>hello world </h1><div>foo bar</div>\n
        <H1>Chapter 2</h1><div>chapter 2 goes here</div>"""

        result = breakIntoPages(html)
        self.assertEqual(result, '<div class="chapter sp-page"><h1>hello world </h1><div>foo bar</div></div>\n'\
                                 '<div class="chapter"><H1>Chapter 2</h1><div>chapter 2 goes here</div></div>')

    def testRemoveLink(self):
        result = removeLinks(html)
        self.assertEqual('<span>HEISE</span>' in result, True)
        self.assertEqual('<span>Control Panel</span>' in result, True)
        self.assertEqual('<span>foo bar</span>' in result, True)




def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ManipulationTests))
    return suite




##########################################################################
# SmartPrintNG - high-quality export of Plone content to
# PDF, RTF, ODT, WML and DOCX
#
# (C) 2007, ZOPYX Ltd & Co. KG, Tuebingen, Germany
##########################################################################

"""
Adapters for specific Plone content types (basicially used
for demonstration purposes.
"""

from zope.interface import implements
from zope.component import adapts

from Products.SmartPrintNG.interfaces import IHTMLExtractor 
from Products.ATContentTypes.interface.folder import IATFolder


class ATFolderMultiplePagesAdapter(object):
    """ ATFolder adapter that generates the concatenation of all documents within
        the current folder.
    """

    adapts(IATFolder)
    implements(IHTMLExtractor)

    def __init__(self, context):
        self.context = context

    def getHTML(self, context):
        """ Return all Documents inside the current folder (not from any subtrees)
            as HTML snippet.
        """

        div_start = '<div class="chapter sp-page">'
        div_start2 = '<div class="chapter">'
        div_end = '</div>'

        lst = []
        brains =  self.context.portal_catalog(portal_type='Document',
                                              path={'query' : '/'.join(self.context.getPhysicalPath()), 'level' : 0},
                                              sort_on='getObjPositionInParent',
                                              )
        num_brains = len(brains)
        for i, brain in enumerate(brains):
            doc = brain.getObject()
            lst.append( (i != num_brains - 1 and div_start or div_start2) + 
                        doc.getText() +
                        div_end)

        return '\n'.join(lst)            



##########################################################################
# SmartPrintNG - high-quality export of Plone content to
# PDF, RTF, ODT, WML and DOCX
#
# (C) 2007, ZOPYX Ltd & Co. KG, Tuebingen, Germany
##########################################################################

"""'
ZCML directives for SmartPrintNG
"""

import os
import logging

from zope.interface import Interface
from zope.schema import TextLine 
import zope.configuration.fields


registry = {}                   
LOG = logging.getLogger()


class ISmartPrintNGResourceDirectory(Interface):
    """ Used for specifying SmartPrintNG resources """

    directory = TextLine(
        title=u"Directory name",
        description=u'Directory path containing templates and styles for a given interface',
        default=u"",
        required=True)

    interfaces = zope.configuration.fields.Tokens(
        title=u"For interfaces",
        description=u'Register resource directory for these interfaces',
        required=True,
        value_type=zope.configuration.fields.GlobalInterface())


def resourceDirectory(_context, directory, interfaces):

    # path of ZCML file
    zcml_filename = _context.info.file

    for iface in interfaces:

        if registry.has_key(iface):
            raise ValueError('%s has already a configured resources directory' % iface)

        # directory name relative to the ZCML directory
        dirname = os.path.join(os.path.dirname(zcml_filename), directory)
        if not os.path.exists(dirname):
            raise IOError('SmartPrintNG resource directory %s does not exist' % dirname)

        d = {'templates' : [], 'stylesheets' : [], 'fo_stylesheets' : []}

        map = {'.pt' : 'templates', 
               '.css' : 'stylesheets', 
               '.fo_css' : 'fo_stylesheets'}

        # scan for templates and stylesheets
        for fname in os.listdir(dirname):

            filename = os.path.join(dirname, fname)
            if not os.path.isfile(filename):
                continue
            basename, ext = os.path.splitext(fname)
            key = map.get(ext)
            if key is not None:
                d[key].append((basename, filename))
            else:
                LOG.warn('Unsupported file type found: %s' % filename)
                continue

        # register resources for interface only if there is something
        # to  be registered
        if d['templates'] or d['stylesheets']:                
            registry[iface] = d

